#ifndef __BIOSTEST_H_INCLUDE_GUARD__
#define __BIOSTEST_H_INCLUDE_GUARD__
/* 
 * Copyright (C) 2006, Intel Corporation
 * 
 * This file is part of the Linux-ready Firmware Developer Kit
 *
 * This program file is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation;version 2.1 of the License.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program in a file named COPYING; if not, write to the 
 * Free Software Foundation, Inc., 
 * 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA
 */


#include <stdint.h>
#include <glib.h>

#include <firmwarekit.h>

#define E820_UNKNOWN	0
#define E820_RESERVED	1
#define E820_ACPI	2
#define E820_USABLE	16

struct detailed_result;

struct major_test {
	char 	*ID;			/* the internal identifier for a test */
	char	*name;			/* user visible name of the test */
	int	result;

	GList 	*details;		/* list of struct detailed_result */

	struct detailed_result *description;
};


struct detailed_result {
	int result;
	char *summary;
	char *details;	
	char *devuri;
};

struct resource_info {
	char *resource;
	char *description;
	char *parent;
};

typedef void voidvoidfunc(void);


struct interactive_test {
	char *shortname;
	voidvoidfunc *execute;
};

extern GList *interactive_tests;
extern GList *hardware_pokes;
extern int finished_tests;
extern int safe_mode;
extern int interactive_mode;


/* dsdt.c */
extern void get_dsdt_resources();

/* dmesg.c */

extern void get_dmesg_buffer(void);
extern void dmesg_common_errors(char *line, char *suggested_uri);
extern void check_dmesg_updates(char *uri);

/* tests.c */

extern void dump_xml_results(void);
extern void dump_results(void);
extern void show_results(void);
extern void sort_results(void);

extern void register_interactive_test(char *name, voidvoidfunc *func);
extern void register_hw_poke(char *name, voidvoidfunc *func);

/* plugins.c */
extern void execute_plugins(char *directory);

/* ui.c */

extern void init_results_ui(void);
extern void report_testrun_ui(char *text);
extern void show_results_ui(GList *list);
extern void simple_ok_dialog(char *title, char *info);
extern void non_fatal_bug(char *info);


/* uri.c */
extern char *get_resource_info(char *uri);
extern struct resource_info *get_resource_struct(char *uri);

extern void dump_xml_resources(char *filename);

/* dumpxml.c */
extern void dump_xml(GList *all_tests, char *filename);
extern void dump_text(GList *all_tests, char *filename);

/* usb.c */
extern void save_on_usb(void);

/* ssh.c */

extern void save_via_ssh(void);
extern int copy_via_ssh(const char *dest);

/* serial.c */

extern void init_serial(int enable, int tty, unsigned long speed);


/* e820.c */
extern void register_e820_line(uint64_t start, uint64_t end, int type);
extern int e820_type(uint64_t memory);
extern void e820_register_resource(void);

/* acpitable.c */
extern unsigned long RSDP_ADDRESS; 
extern void extract_dsdt_ssdts();
extern int locate_acpi_table(char *name, unsigned long *address, unsigned long *size);
extern int locate_acpi_table32(char *name, unsigned long *address, unsigned long *size);
extern int locate_acpi_table64(char *name, unsigned long *address, unsigned long *size);
extern int e820_is_reserved(uint64_t memory);
extern char *copy_acpi_table(uint64_t address, char *name);
extern char *execute_aml_method(char *method, char *execute);

/* lib.c */

extern void chop_newline(char *s);
extern char *scatprintf(char *buffer, const char *fmt, ...);
extern void chop_lastchar(char *s);
extern int is_prefix(char *needle, char *haystack);
extern char *xmlescape(char *string);
extern int space_count(char *s);


#endif

