/* 
 * Copyright (C) 2006, Intel Corporation
 * 
 * This file is part of the Linux-ready Firmware Developer Kit
 *
 * This program file is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation; version 2.1 of the License.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program in a file named COPYING; if not, write to the 
 * Free Software Foundation, Inc., 
 * 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA
 */

#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <stdarg.h>

#include <glib.h>

#include "biostest.h"


/* Dump all test results in an xml formatted file */
void dump_xml(GList *all_tests, char *filename)
{
	FILE *file;
	GList *list1, *list2;
	struct major_test *test;
	struct detailed_result *result;

	file = fopen(filename, "w");
	if (!file)
		return;

	
	fprintf(file, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
	fprintf(file, "<?xml-stylesheet href=\"results.css\" type=\"text/css\"?>\n");
	fprintf(file, "<results>\n");	

	
	list1 = g_list_first(all_tests);
	while (list1) {
		test = (struct major_test *)list1->data;

		fprintf(file,"<test>\n\t<id>%s</id>\n\t<name>%s</name>\n\t<result>%i</result>\n \n", 
				test->ID, test->name, test->result);
		if (test->description)
			fprintf(file, "\t<description>%s</description>\n", test->description->details);
		

		list2 = g_list_first(test->details);
		while (list2) {
			result = (struct detailed_result*)list2->data;			
			fprintf(file,"\t<detail>\n\t\t<summary>%s</summary>\n\t\t<result>%i</result>\n",
				result->summary, result->result);
			if (result->devuri)
				fprintf(file,"\t\t<uri>%s</uri>\n\t\t", result->devuri);
			if (result->details) {
				char *c;
				c = xmlescape(result->details);
				fprintf(file,"\t\t<data>%s</data>\n", c);
				free(c);
			}
			fprintf(file,"\t</detail>\n");


			list2 = g_list_next(list2);
		}
		fprintf(file,"</test>\n");

		list1 = g_list_next(list1);
	}
	fprintf(file, "</results>\n");	
	fclose(file);
}

/* Dump all test results in a text formatted file */
void dump_text(GList *all_tests, char *filename)
{
	FILE *file;
	GList *list1, *list2;
	struct major_test *test;
	struct detailed_result *result;
	int nrtotal, nrpass, nrwarn, nrfail;

	file = fopen(filename, "w");
	if (!file)
		return;

	fprintf(file, "-------------------------------------------------\n"
			"  Date: %s                                       \n"
			"*                                               *\n"
			"* Firmwarekit (release 2.0)                     *\n"
			"* http://www.linuxfirmwarekit.org               *\n"
			"*                                               *\n"
			"*                                               *\n"
			"* For more information on test descriptions     *\n"
			"* and details on what the PASS/INFO/WARN/FAIL   *\n"
			"* results mean, go to: Documentation/TestsInfo. *\n"
			"*                                               *\n"
			"-------------------------------------------------\n\n", 
			get_time_stamp());

	
	list1 = g_list_first(all_tests);

	/* Firt get summary of test report */
	nrtotal = 0;
	nrpass = 0;
	nrfail = 0;
	nrwarn = 0;

	while (list1) {
		test = (struct major_test *)list1->data;

		nrtotal++;
		switch (test->result) {
		case	PASS:
		case	INFO:
			nrpass++;
			break;
		case 	WARN:
			nrwarn++;
			break;
		case	FAIL:
			nrfail++;
			break;
		}

		list1 = g_list_next(list1);
	}
	fprintf(file, "SUMMARY:  %d Fails, %d Warns, %d Pass, %d Total\n\n", 
			nrfail, nrwarn, nrpass, nrtotal);
	
	list1 = g_list_first(all_tests);
	while (list1) {
		test = (struct major_test *)list1->data;

		
		fprintf(file, "=================================================\n"
			"* Plugin name: %s\n\n"
			"* Title: %s\n"
			"* Description: %s\n"
			"* Result: %s\n"
			"================================================\n\n", 
			test->ID, test->name, test->description->details, 
			result_to_ascii(test->result));


		list2 = g_list_first(test->details);
		while (list2) {
			result = (struct detailed_result*)list2->data;	

			fprintf(file, "[%s]-%s\n\n", result_to_ascii(result->result), 
					result->summary);
			if (result->details)
				fprintf(file, "%s\n\n", result->details);

			list2 = g_list_next(list2);
		}
		list1 = g_list_next(list1);
	}
	fclose(file);
}
