/* 
 * Copyright (C) 2006, Intel Corporation
 * 
 * This file is part of the Linux-ready Firmware Developer Kit
 *
 * This program file is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation;version 2.1 of the License.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program in a file named COPYING; if not, write to the 
 * Free Software Foundation, Inc., 
 * 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA
 */

#define _GNU_SOURCE 
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
#include <time.h>

#include <slang.h>

#include <glib.h>

#include "biostest.h"


char *scatprintf(char *buffer, const char *fmt, ...)
{
	char *vasprintf_buffer;
	char *new;

	/* I'd like to just push buffer as first argument, and add a %s to fmt, but alas */
	va_list argp;
	va_start(argp, fmt);
	vasprintf (&vasprintf_buffer, fmt, argp);
	va_end(argp);

	if (!buffer) 
		return vasprintf_buffer;

	asprintf(&new, "%s%s", buffer, vasprintf_buffer);
	free(vasprintf_buffer);
	free(buffer);

	return new;
}

void chop_newline(char *s)
{
	if (!s)
		return;
	while (strlen(s)>0 && s[strlen(s)-1]=='\n')
		s[strlen(s)-1]=0;
}

void chop_lastchar(char *s)
{
	if (!s)
		return;
	if (strlen(s)<1)
		return;
	s[strlen(s)-1]=0;
}


int is_prefix(char *needle, char *haystack)
{
	if (strlen(needle) > strlen(haystack))
		return 0;

	if (strncmp(needle, haystack, strlen(needle))==0)
		return 1;
	return 0;
}


char *xmlescape(char *string)
{
	char *buffer;
	char b[2];
	char *c,*d;

	b[1]=0;

	if (!string)
		return 0;
	if (strlen(string)==0)
		return strdup("");

	buffer = malloc(5 * strlen(string) + 1);
	if (!buffer)
		return NULL;
	buffer[0] = 0;
	d = buffer;
	c = string;

	while (c && *c) {
		if (*c == '<') {
			strcpy(d,"&lt;");
			d+=4;
		} else if (*c == '>') {
			strcpy(d,"&gt;");
			d+=4;
		} else if (*c == '&') {
			strcpy(d,"&amp;");
			d+=5;
		} else {
			b[0] = *c;
			strcpy(d, b);
			d++;
		}
		c++;
			
	}
	return buffer;
}

int space_count(char *s)
{
	int count = 0;

	if (!s)
		return 0;

	while (*s==' ') {
		count++;
		s++;
	}
	return count;
}

void AML_to_uri(char *aml, char *uri)
{
	char *c;
	*uri = 0;
	int dotcount = 0;
	c = uri;
	c += sprintf(uri, "aml://");
	while (aml && *aml) {
		if (*aml=='.') {
			while (dotcount<4) {
				*c = '_';
				c++;
				dotcount++;	
			}
			*c = '.';
			c++;
			dotcount = 0;
		} else { 
			*c = *aml;
			c++;
			if (*aml != '\\') 
				dotcount++;
		}
		aml++;
	}
	*c = 0;
}

char *get_time_stamp(void) 
{
	char *time_stamp;
	const time_t cur_time = time(NULL);

	struct tm *loc_time = localtime(&cur_time);

	if (loc_time == NULL)
		return strdup("");
	else {
		time_stamp = asctime(loc_time);		
		chop_newline(time_stamp);
		return strdup(time_stamp);
	}
	
}

char *result_to_ascii(int res) {

	switch (res) {
	case PASS:	
		return strdup("PASS");
	case WARN:
		return strdup("WARN");
	case INFO:
		return strdup("INFO");
	case FAIL:
		return strdup("FAIL");
	default:
		return strdup("??");
	}
}
