/* 
 * Copyright (C) 2006, Intel Corporation
 * 
 * This file is part of the Linux-ready Firmware Developer Kit
 *
 * This program file is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation;version 2.1 of the License.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program in a file named COPYING; if not, write to the 
 * Free Software Foundation, Inc., 
 * 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA
 */



#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


#include <linux/vt.h>
#include <sys/ioctl.h>
#include <termios.h>

#include <glib.h>

#include "biostest.h"

void init_serial(int enable, int tty, unsigned long speed)
{
	int fd;	
	struct termios termios;
	struct winsize winsize;
	char dev_name[255];
	char cp_command[255];

	sprintf(dev_name, "/dev/tty%d", tty);
	sprintf(cp_command, "cp -a %s /dev/tty", dev_name);

	memset(&winsize, 0, sizeof(winsize));
	memset(&termios, 0, sizeof(termios));

	/* debug: don't mess with /dev */
	if (getuid()!=0)
		return;

	/* root but not in the artificial environment */
	if (access("/dev/tty.console", R_OK))
		return;

	/* if we're not using serial output */
	if (!enable) {
		unlink("/dev/tty");
		if (system("cp -a /dev/tty.console /dev/tty"))
			return;
		return;
	}

	fprintf(stderr, "Switching to serial ports for application output.\n");
	fprintf(stderr, "Remember to set your terminal settings to 115200/8N1.\n");


	fd = open(dev_name, O_RDWR);

	if (fd<0)
		return;

	unlink("/dev/tty");
	if (system(cp_command))
		return;

	tcgetattr(fd, &termios);

	
	/* we basically want a raw serial connection */
	termios.c_iflag  = 0;
	termios.c_oflag  = 0;
	termios.c_lflag  = 0;
	termios.c_cc[VMIN]  = 1;
	termios.c_cc[VTIME] = 0;

	cfsetospeed(&termios , speed);
	cfsetispeed(&termios , speed);
	
	tcsetattr(fd, TCSANOW, &termios);

	/* serial "screen" is 80x24, because we need to leave one for the status bar of $@$!@ minicom */

	winsize.ws_col = 80;
	winsize.ws_row = 24;


	if (ioctl(fd, TIOCSWINSZ, &winsize)) 
        	fprintf(stderr, "Failed to set the screen size");

	close(0);
	close(1);
	dup2(fd, 0);
	dup2(fd, 1);
	close(fd);
}
