
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>

#include <newt.h>

#include "biostest.h"
#define MAX_SSH_DEST_LEN 256
char ssh_dest[MAX_SSH_DEST_LEN]="host_ip:/target_dir";

#define MAX_SSH_COMMAND_LEN MAX_SSH_DEST_LEN + 512 + HOST_NAME_MAX

void setup_result_dir(char *result_dir, size_t string_sz){

	char host_name[HOST_NAME_MAX + 1];
	int ret;

	char date_str[128];
	const time_t cur_time = time(NULL);

	struct tm *loc_time = localtime(&cur_time);

	if (loc_time == NULL)
		strcpy (date_str, "");
	else
		sprintf(date_str, "%.2d%.2d%.2d%.2d%.2d", loc_time->tm_hour,
			loc_time->tm_min, loc_time->tm_mday, loc_time->tm_mon + 1,
			loc_time->tm_year % 100);

	ret = gethostname(host_name, HOST_NAME_MAX);
	host_name[HOST_NAME_MAX] = '\0';

	/* gethostname may return "(none)" and no error return value in
	   some cases? There might be other invalid fs character used as host
	   name? I don't care for that now...
	   -> handle it
	*/
	if (ret || strstr(host_name, "(") || strstr(host_name, ")")){
		snprintf(result_dir, string_sz-1, "results_%s", date_str);
		result_dir[string_sz-1] = '\n';
	}
	else{
		snprintf(result_dir, string_sz-1, "results_%s_%s", host_name, date_str);
		result_dir[string_sz-1] = '\n';
	}
	return;
}

int copy_via_ssh(const char *dest)
{
	char command[4096];
	char result_dir[1024];
	int ret1, ret2;
	setup_result_dir(result_dir, 1024);

	/* Check and create tmp dir */
	snprintf (command, 4095, "/var/log/firmwarekit/%s", result_dir);
	if (access(command, W_OK)){
		ret1 = snprintf (command, 4095, "mkdir -p /var/log/firmwarekit/%s",
				 result_dir);
		ret2 = system(command);
		if (ret1 >= 4096 || ret2)
			goto error;
	}

	/* Copy results into tmp dir */
	ret1 = snprintf (command, 4096, "cp /var/log/firmwarekit/acpi.dump "
			"/var/log/firmwarekit/DSDT* /var/log/firmwarekit/results.* "
			"/var/log/firmwarekit/resources.* "
			"/var/log/firmwarekit/%s", result_dir);

	ret2 = system(command);
	if (ret1 >= 4096 || ret2)
		goto error;

	/* secure copy stuff away */
	ret1 = snprintf (command, 4096, "scp -r -o CheckHostIP=no -o "
			"StrictHostKeyChecking=no /var/log/firmwarekit/%s"
			" %s", result_dir, dest);
	ret2 = system(command);
	if (ret1 >= 4096 || ret2)
		goto error;
	return 0;
 error:
	fprintf (stderr, "WARN: ssh copying did not succeed. ret1: %d, "
		 "ret2: %d\n", ret1, ret2);
	return 1;
}

void save_via_ssh(void)
{
	int myWin;
	newtComponent myForm;

	newtComponent mySave, myBack, mySCPDest, resu;

	const char *destination;
	int ret;

	int W,H;

	newtGetScreenSize(&W,&H);

	myWin = newtOpenWindow(1+(W-80)/2,1+(H-10)/2,80,10, "Provide"
			       "destination where to copy results"
			       " in scp syntax");
	myForm = newtForm(NULL,NULL,0);

	newtFormAddComponent(myForm, newtLabel(2,2, "Destination"));
	mySCPDest = newtEntry(15, 2, ssh_dest, MAX_SSH_DEST_LEN, &destination, 0);
	newtFormAddComponent(myForm, mySCPDest);

/* Password passing is not possible via scp...
  	mySCPDest = newtEntry(20, 4, "host:/target_dir", 20, &pass, NEWT_FLAG_HIDDEN);
	newtFormAddComponent(myForm, mySCPDest);
*/
	mySave = newtButton(2, 4, "Save");
        newtFormAddComponent(myForm, mySave);

	myBack = newtButton(27, 4, "Back");
        newtFormAddComponent(myForm, myBack);

	resu = newtRunForm(myForm);
	if (resu == mySave) {
		newtCls();
		ret = copy_via_ssh(destination);
		if (ret)
			simple_ok_dialog("SCP result", "ERROR: scp command did"
					 " not succeed");
		else
			simple_ok_dialog("SCP result", "Copying data was "
					 "successfull");
	}
	newtPopWindow();
}
