/* 
 * Copyright (C) 2006, Intel Corporation
 * 
 * This file is part of the Linux-ready Firmware Developer Kit
 *
 * This program file is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation;version 2.1 of the License.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program in a file named COPYING; if not, write to the 
 * Free Software Foundation, Inc., 
 * 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <unistd.h>

#include <glib.h>
#include <newt.h>

#include "biostest.h"


static int myProgressWin=-1;
static newtComponent myProgressText, myProgressForm, myScales;

static GList *save_pokes;

static void register_save_poke(char *name, voidvoidfunc *func)
{
	struct interactive_test *test;
	test = malloc(sizeof(struct interactive_test));
	assert(test!=NULL);
	memset(test, 0, sizeof(struct interactive_test));
	test->shortname = strdup(name);
	test->execute = func;
	save_pokes = g_list_append(save_pokes, test);
}

static int longest_string(char *str)
{
	char *c;
	int longest=0;
	while (strlen(str)>0) {
		c = strchr(str,'\n');
		if (!c) break;
		if (longest < c-str)
			longest = c-str;
		str=c+1;
	}
	return longest;
}

/* Display LFDK root window and welcome screen */
void init_results_ui(void)
{
	int myHelloWin;
	newtComponent myHelloText, myHelloForm;
	newtInit();
	newtCls();
	newtDrawRootText(0,0, " Linux-ready Firmware Developer Kit -  Release 2  - (C) 2007 Intel Corporation");
	newtPushHelpLine(     "              Linux-ready Firmware Developer Kit");

	int W,H;
	newtGetScreenSize(&W,&H);

	myHelloWin = newtOpenWindow(5,3,W-10,H-7, "Welcome to the Linux-ready Firmware Developer Kit");
	myHelloForm = newtForm(NULL,NULL,0);
	myHelloText = newtTextbox(4,1,W-16,H-8, 0);
	newtFormAddComponent(myHelloForm, myHelloText);

	newtTextboxSetText(myHelloText, 
		"This toolkit is created for Firmware developers, Firmware \n"
		"testers, support personnel and Linux* kernel developers to help \n"
		"validate Firmware and the interaction with the Linux operating\n"
		"system. \n\n"
		"           Copyright (C) 2007 Intel Corporation      \n\n"
		"This program is free software; you can redistribute it and/or \n"
		"modify it under the terms of the GNU Lesser General Public \n"
		"License version 2.1 as published by the Free Software Foundation\n\n"
		"             http://www.linuxfirmwarekit.org         \n\n"
		"Intel is a registered trademark of Intel Corporation \n"
		"*Other names and brands may be claimed as the property of others"
		);

	newtDrawForm(myHelloForm);
	newtRefresh();		
	
	if (getuid()==0)
	sleep(6);  /* the kernel tends to spew for a few seconds or so after boot which
		      distords the screen. Just show this text during this time to
		      not have the real app run with a distorted screen.
	            */
	newtPopWindow();
}


/* Display LFDK help window */
void help_screen(void)
{
	int myHelloWin;
	newtComponent myHelloText, myHelloForm;
	newtComponent myOkButton;
	int W,H;
	newtGetScreenSize(&W,&H);

	myHelloWin = newtOpenWindow(7,3,W-14,H-7, "Brief Help Screen");
	myHelloForm = newtForm(NULL,NULL,0);
	myHelloText = newtTextbox(4,1,W-20,H-8, 0);
	newtFormAddComponent(myHelloForm, myHelloText);

	newtTextboxSetText(myHelloText, 
		"More extensive documentation is available at: \n\n"
		"             http://www.linuxfirmwarekit.org         \n\n"
		"To navigate this program, use the <TAB> and the cursor\n"
		"keys to switch between the buttons and output lines.\n\n"
		"Press <ENTER> to get more info on the test [IN BRACKETS] \n"
		"or on a specific result line. This opens a dialog with\n"
		"a detailed explanation."

		);

	myOkButton = newtButton(3,H-11, "OK");
	newtFormAddComponent(myHelloForm, myOkButton);

	newtRunForm(myHelloForm);
	newtPopWindow();
}

/* Initializing test progress window */
void report_testrun_ui(char *text)
{
	int W,H;
	newtGetScreenSize(&W,&H);

	/* If this is the first test being run, show 'performing tests'
	 * message first. */
	if (myProgressWin==-1) {
		myProgressWin = newtOpenWindow(5,9,W-12,7, "Performing tests");
		myProgressForm = newtForm(NULL,NULL,0);
		myProgressText = newtTextbox(3,2,W-20,1, 0);
		myScales = newtScale(3,5,W-19, 100);
		newtFormAddComponent(myProgressForm, myProgressText);
		newtFormAddComponent(myProgressForm, myScales);
	}

	newtTextboxSetText(myProgressText, text);

	newtDrawForm(myProgressForm);
	newtRefresh();		
}

/* Display progress bar */
void report_testrun_progress(int percentage)
{
	/* abort when things aren't initialized yet */
	if (myProgressForm==myScales || myProgressWin==-1)
		return;
	newtScaleSet(myScales, percentage);
	newtRefresh();
}	


static void show_resource_info(char *uri)
{
	int W,H;
	int win;
	newtComponent myForm;
	newtComponent myOkButton, myParentButton;
	newtComponent myText;
	newtComponent resu;
	int WP,HP, NW, NH;
	char *reflow;
	int len, flag;
	struct resource_info *res;


	res = get_resource_struct(uri);

	if (res==NULL)
		return;

	if (res->description == NULL)
		return;

	newtGetScreenSize(&W,&H);

	WP=W-14;
	HP=H-11;

	reflow = newtReflowText(res->description, WP, 0, 0, &NW, &NH);

	len = longest_string(reflow);
	if (len<NW)
		NW=len;

	if (NW < WP)
		WP=NW;
	if (NH < HP)
		HP=NH;

	if (WP<15) WP=24;
	if (HP<4) HP=4;


	win = newtOpenWindow((W-(WP+4))/2,(H-(HP+7))/2,WP+4,HP+7, "Resource Information");

	myForm = newtForm(NULL, NULL, 0);

	myOkButton = newtButton(2,HP+3, "Back");
	newtFormAddComponent(myForm, myOkButton);

	myParentButton = newtButton(11,HP+3, "Parent");
	if (res->parent && strcmp(res->parent, res->resource)!=0 && get_resource_struct(res->parent)!=NULL)
		newtFormAddComponent(myForm, myParentButton);

	flag = 0;
	if (NH>HP)
		flag = NEWT_FLAG_SCROLL;

	myText = newtTextbox(2,1,WP,HP, flag);

	newtFormAddComponent(myForm, myText);

	newtTextboxSetText(myText,reflow);
	free(reflow);
	
	while (1) {
		resu = newtRunForm(myForm);
		if (resu == myParentButton)
			show_resource_info(res->parent);
		else
			break;
	}
	newtPopWindow();
}

void simple_ok_dialog(char *title, char *info)
{
	int W,H;
	int win;
	newtComponent myForm;
	newtComponent myOkButton;
	newtComponent myText;
	int WP,HP, NW, NH;
	char *reflow;
	int len, flag;

	if (info==NULL || title==NULL)
		return;

	newtGetScreenSize(&W,&H);

	WP=W-14;
	HP=H-11;

	reflow = newtReflowText(info, WP, 0, 0, &NW, &NH);

	len = longest_string(reflow);
	if (len<NW)
		NW=len;

	if (NW < WP)
		WP=NW;
	if (NH < HP)
		HP=NH;

	if (WP<15) WP=15;
	if (HP<4) HP=4;


	win = newtOpenWindow((W-(WP+4))/2,(H-(HP+7))/2,WP+4,HP+7, title);

	myForm = newtForm(NULL, NULL, 0);

	myOkButton = newtButton(2,HP+3, "Back");
	newtFormAddComponent(myForm, myOkButton);

	flag = 0;
	if (NH>HP)
		flag = NEWT_FLAG_SCROLL;

	myText = newtTextbox(2,1,WP,HP, flag);

	newtFormAddComponent(myForm, myText);

	newtTextboxSetText(myText,reflow);
	free(reflow);
	
	newtRunForm(myForm);
	newtPopWindow();
}

void non_fatal_bug(char *info)
{
	int W,H;
	int win;
	newtComponent myForm;
	newtComponent myOkButton;
	newtComponent myText;
	int WP,HP, NW, NH;
	char *reflow;
	int len, flag;

	if (info==NULL)
		return;

	newtGetScreenSize(&W,&H);

	WP=W-14;
	HP=H-11;

	reflow = newtReflowText(info, WP, 0, 0, &NW, &NH);

	len = longest_string(reflow);
	if (len<NW)
		NW=len;

	if (NW < WP)
		WP=NW;
	if (NH < HP)
		HP=NH;

	if (WP<15) WP=15;
	if (HP<4) HP=4;


	win = newtOpenWindow((W-(WP+4))/2,(H-(HP+7))/2,WP+4,HP+7, "Non-fatal bug");

	myForm = newtForm(NULL, NULL, 0);

	myOkButton = newtButton(2,HP+3, "OK");
	newtFormAddComponent(myForm, myOkButton);

	flag = 0;
	if (NH>HP)
		flag = NEWT_FLAG_SCROLL;

	myText = newtTextbox(2,1,WP,HP, flag);

	newtFormAddComponent(myForm, myText);

	newtTextboxSetText(myText,reflow);
	free(reflow);
	
	newtDrawForm(myForm);
	newtRefresh();		
	
	sleep(6);  
	newtPopWindow();
}

static void show_result(struct detailed_result *result)
{
	int W,H;
	int WP,HP, NW, NH;
	char *c, *reflow;
	int win;
	newtComponent myForm;
	newtComponent myOkButton;
	newtComponent myResourceButton;
	newtComponent myText;
	newtComponent inp;
	int flag;
	int len;

	newtGetScreenSize(&W,&H);



	c= get_resource_info(result->devuri);

	if (result->details==NULL && c==NULL && result->summary == NULL)
		return;

	if (result->details==NULL)
		result->details = c;

	if (result->details == NULL)
		result->details = result->summary;

	WP=W-13;
	HP=H-11;

	

	reflow = newtReflowText(result->details, WP, 0 , 0, &NW, &NH);

	len = longest_string(reflow);
	if (NW>len)
		NW=len;

	if (NW < WP)
		WP=NW;
	if (NH < HP)
		HP=NH;

	if (WP<15) WP=15;
	if (WP<25 && c && c!=result->details)
		     WP=25;
	if (HP<4) HP=4;


	win = newtOpenWindow((W-(WP+4))/2,(H-(HP+7))/2,WP+4,HP+7, "Detailed Results");

	myForm = newtForm(NULL, NULL, 0);

	myOkButton = newtButton(2,HP+3, "Back");
	newtFormAddComponent(myForm, myOkButton);

	myResourceButton = newtButton(12,HP+3, "Resource info");
	if (c && c!=result->details) {
		newtFormAddComponent(myForm, myResourceButton);
	}

	if (NH>HP) 
		flag = NEWT_FLAG_SCROLL;
	else
		flag = 0;

	myText = newtTextbox(2,2,WP,HP, flag);

	newtFormAddComponent(myForm, myText);

	newtTextboxSetText(myText, reflow);
	free(reflow);
	
	while (1) {
		inp = newtRunForm(myForm);
		if (inp == myOkButton)
			break;
		if (inp == myResourceButton)
			show_resource_info(result->devuri);
	}
	newtPopWindow();
}

static void int_poke_dialog(char *title, GList *list) 
{
	int win;

	newtComponent myForm;
	newtComponent myOkButton;
	newtComponent myList;
	newtComponent resu;
	int W,H;
	int w,h;
	int flags = 0;

	GList *item;
	struct interactive_test *test;

	newtGetScreenSize(&W,&H);

	w=0;
	h=0;
	item = g_list_first(list);
	while (item) {
		test = (struct interactive_test*) item->data;
		h++;
		if (strlen(test->shortname)> (unsigned int)w)
			w = strlen(test->shortname);
		item = g_list_next(item);
	}

	flags = NEWT_FLAG_RETURNEXIT;
	if (h>H-10) {
		h = H-10;
		flags |= NEWT_FLAG_SCROLL;
	}

	if (w>W-10)
		w = W-10;
	

	win = newtOpenWindow( (W-(w+6))/2,(H-(h+6))/2,w+6,h+6, title);
	myForm = newtForm(NULL, NULL, 0);

	myList = newtListbox(2,1,h, flags);
	newtFormAddComponent(myForm, myList);
	newtListboxSetWidth(myList, w+4);
	
	item = g_list_first(list);
	while (item) {
		test = (struct interactive_test*) item->data;

		newtListboxAppendEntry(myList, strdup(test->shortname), test);
		item = g_list_next(item);
	}

	myOkButton = newtButton(2,h+2, "Back");
	newtFormAddComponent(myForm, myOkButton);

	while (1) {
		resu = newtRunForm(myForm);
		if (resu == myList) {
			test = newtListboxGetCurrent(myList);
			if (test) 
				test->execute();			
			else 
				break;
		} else 
			break;
	}

	newtPopWindow();
	newtFormDestroy(myForm);
}

static int nrtotal, nrpass, nrfail, nrwarn;

static void count_results(GList *all_tests)
{
	GList *list1;
        struct major_test *test;

	nrtotal = 0;
	nrpass = 0;
	nrfail = 0;
	nrwarn = 0;

	list1 = g_list_first(all_tests);
	while (list1) {
		test = (struct major_test *)list1->data;

		nrtotal++;
		switch (test->result) {
		case	PASS:
		case	INFO:
			nrpass++;
			break;
		case 	WARN:
			nrwarn++;
			break;
		case	FAIL:
			nrfail++;
			break;
		}

		list1 = g_list_next(list1);
	}
}

static void fill_with_results(GList *all_tests, newtComponent myList)
{
	char *buffer;
	GList *list1, *list2;
        struct major_test *test;
        struct detailed_result *result;

	nrtotal = 0;
	nrpass = 0;
	nrfail = 0;
	nrwarn = 0;

	newtListboxClear(myList);
	list1 = g_list_first(all_tests);
	while (list1) {
		test = (struct major_test *)list1->data;

		buffer = strdup("");

		nrtotal++;
		switch (test->result) {
		case	PASS:
			buffer = scatprintf(buffer, "[PASS] ");	
			nrpass++;
			break;
		case	INFO:
			buffer = scatprintf(buffer, "[INFO] ");
			nrpass++;
			break;
		case 	WARN:
			buffer = scatprintf(buffer, "[WARN] ");
			nrwarn++;
			break;
		case	FAIL:
			buffer = scatprintf(buffer, "[FAIL] ");
			nrfail++;
			break;
		default:	
			buffer = scatprintf(buffer, "[UNKN] ");
		}

		buffer = scatprintf(buffer, "%s \n", test->name);
		newtListboxAppendEntry(myList, strdup(buffer), test->description);
		free(buffer);

		list2 = g_list_first(test->details);
		while (list2) {
			char buf[4096];
			char c;
			result = (struct detailed_result*)list2->data;			

			switch (result->result) {
			case	PASS:
				c=' ';
				break;
			case	INFO:
				c=' ';
				break;
			case 	WARN:
				c='W';
				break;
			case	FAIL:
				c='F';
				break;
			default:	
				c='U';
				break;
			}
			sprintf(buf, "  %c       %s\n", c,result->summary);

			newtListboxAppendEntry(myList, strdup(buf), result);

			list2 = g_list_next(list2);
		}

		list1 = g_list_next(list1);
	}
}

/* Display LFDK results window, the one you see after all the
 * tests have been run, and the results are displayed. */
void show_results_ui(GList *all_tests)
{
	int win;
	newtComponent myForm;
	newtComponent myOkButton;
	newtComponent mySaveButton;
	newtComponent myIntButton;
	newtComponent myPokeButton;
	newtComponent myHelpButton;
	newtComponent myList;
	newtComponent resu;
        struct detailed_result *result;
	char buffer[256];

	int W,H;

	newtPopWindow();


	newtGetScreenSize(&W,&H);

	count_results(all_tests);

	sprintf(buffer,"Test results: %i fail, %i warn, %i pass, %i total", nrfail, nrwarn, nrpass, nrtotal);

	win = newtOpenWindow(1,2,W-3,H-4, buffer);


	myForm = newtForm(NULL, NULL, 0);



	myList = newtListbox(2,1,H-10, NEWT_FLAG_SCROLL | NEWT_FLAG_RETURNEXIT);

	newtFormAddComponent(myForm, myList);

	newtListboxSetWidth(myList, W-6);

	fill_with_results(all_tests, myList);


	myOkButton = newtButton(2,H-8, "Exit");
	newtFormAddComponent(myForm, myOkButton);

	mySaveButton = newtButton(12,H-8, "Save");
	newtFormAddComponent(myForm, mySaveButton);

	/* Setup save options */

	register_save_poke("Save to USB", save_on_usb);
	register_save_poke("Save via SSH", save_via_ssh);

	myIntButton = newtButton(22,H-8, "Manual Tests");
	if (interactive_tests)
		newtFormAddComponent(myForm, myIntButton);

	myPokeButton = newtButton(40,H-8, "Poke HW");
	if (hardware_pokes)
		newtFormAddComponent(myForm, myPokeButton);

	myHelpButton = newtButton(54,H-8, "Help");
	newtFormAddComponent(myForm, myHelpButton);

	if (!interactive_mode) {
		save_on_usb();
	} else	while (1) {
		resu = newtRunForm(myForm);
		if (resu == myList) {
			result = newtListboxGetCurrent(myList);
			if (result && result->details)
				show_result(result);			
			else if (result && result->devuri && strlen(result->devuri)>0)
				show_resource_info(result->devuri);
			else
				show_result(result);
		} else if (resu == mySaveButton) {
			int_poke_dialog("Save results", save_pokes);
		} else if (resu == myPokeButton) {
			int_poke_dialog("Poke hardware", hardware_pokes);
		} else if (resu == myIntButton) {
			int_poke_dialog("Manual tests", interactive_tests);
			fill_with_results(all_tests, myList);
		} else if (resu == myHelpButton) {
			help_screen();
		} else 
			break;
	}



	newtRefresh();

	newtPopWindow();
	newtFinished();
}
