/* 
 * Copyright (C) 2006, Intel Corporation
 * 
 * This file is part of the Linux-ready Firmware Developer Kit
 *
 * This program file is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation;version 2.1 of the License.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program in a file named COPYING; if not, write to the 
 * Free Software Foundation, Inc., 
 * 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <assert.h>

#include <sys/klog.h>

#include "biostest.h"


static GList *resources;


void announce_resource(char *uri, char *description, char *parent)
{
	struct resource_info *res;

	res = malloc(sizeof(struct resource_info));
	assert(res!=NULL);
	memset(res, 0, sizeof(struct resource_info));
	res->resource = strdup(uri);
	res->description = strdup(description);
	if (parent)
		res->parent = strdup(parent);
	resources = g_list_append(resources, res);
}

char *get_resource_info(char *uri)
{
	GList *list;
	struct resource_info *res;

	if (uri==NULL)
		return NULL;

	list = g_list_first(resources);
	while (list) {
		res = (struct resource_info*)list->data;
		if (strcmp(res->resource, uri)==0)
			return res->description;
		list = g_list_next(list);
	}
	return NULL;
}

struct resource_info *get_resource_struct(char *uri)
{
	GList *list;
	struct resource_info *res;

	if (uri==NULL)
		return NULL;

	list = g_list_first(resources);
	while (list) {
		res = (struct resource_info*)list->data;
		if (strcmp(res->resource, uri)==0)
			return res;
		list = g_list_next(list);
	}
	return NULL;
}


void dump_xml_resources(char *filename)
{
	FILE *file;
	GList *list;
	struct resource_info *res;
	char *c;

	file = fopen(filename, "w");
	if (!file)
		return;
	fprintf(file, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
	fprintf(file, "<?xml-stylesheet href=\"resources.css\" type=\"text/css\"?>\n");
	fprintf(file, "<resources>\n");
	list = g_list_first(resources);
	while (list) {
		res = (struct resource_info*)list->data;
		fprintf(file, "<resource>\n\t<uri>%s</uri>\n", res->resource);
		c = xmlescape(res->description);
		if (res->parent)
			fprintf(file, "\t<parent>%s</parent>\n", res->parent);
		fprintf(file, "\t<description>%s</description>\n", c);
		free(c);
		fprintf(file, "</resource>\n");
		list = g_list_next(list);
	}
	fprintf(file, "</resources>\n");
	fclose(file);
}
